@@ -40,7 +40,7 @@ import msgparsing
 from diff import seddiff
 
 
-VERSION = '1.21'
+VERSION = '1.22'
 
 
 class IrcDisconnectError(Exception): ...
@@ -67,6 +67,7 @@ class Replies(Enum):
     RPL_ENDOFNAMES = 366
     ERR_NOSUCHNICK = 401
     ERR_NOSUCHCHANNEL = 403
+    ERR_CANNOTSENDTOCHAN = 404
     ERR_UNKNOWNCOMMAND = 421
     ERR_FILEERROR = 424
     ERR_ERRONEUSNICKNAME = 432
@@ -180,7 +181,7 @@ class Client:
         if self.settings.autojoin:
             mpim_cutoff = datetime.datetime.utcnow() - MPIM_HIDE_DELAY
 
-            for sl_chan in await self.sl_client.channels():
+            for sl_chan in await self.sl_client.joined_channels():
                 if not sl_chan.is_member:
                     continue
 
@@ -194,7 +195,7 @@ class Client:
                     continue
                 await self._send_chan_info(channel_name_b, sl_chan)
         else:
-            for sl_chan in await self.sl_client.channels():
+            for sl_chan in await self.sl_client.joined_channels():
                 channel_name = '#%s' % sl_chan.name_normalized
                 self.parted_channels.add(channel_name.encode('utf-8'))
 
@@ -297,20 +298,23 @@ class Client:
 
         message = await self._addmagic(msg.decode('utf8'), dest_object)
 
-        if isinstance(dest_object, slack.User):
-            await self.sl_client.send_message_to_user(
-                dest_object,
-                message,
-                action,
-                re_send_to_irc,
-            )
-        else:
-            await self.sl_client.send_message(
-                dest_object,
-                message,
-                action,
-                re_send_to_irc
-            )
+        try:
+            if isinstance(dest_object, slack.User):
+                await self.sl_client.send_message_to_user(
+                    dest_object,
+                    message,
+                    action,
+                    re_send_to_irc,
+                )
+            else:
+                await self.sl_client.send_message(
+                    dest_object,
+                    message,
+                    action,
+                    re_send_to_irc
+                )
+        except Exception as e:
+            await self._sendreply(Replies.ERR_CANNOTSENDTOCHAN, f'Message sending failed: {e}')
 
     async def _listhandler(self, cmd: bytes) -> None:
         for c in await self.sl_client.channels(refresh=True):
